package betterwithmods.common.blocks.mechanical.tile;

import betterwithmods.api.BWMAPI;
import betterwithmods.api.capabilities.CapabilityMechanicalPower;
import betterwithmods.api.tile.ICrankable;
import betterwithmods.api.tile.IMechanicalPower;
import betterwithmods.common.BWRegistry;
import betterwithmods.common.BWSounds;
import betterwithmods.common.blocks.mechanical.BlockMechMachines;
import betterwithmods.common.blocks.tile.TileBasicInventory;
import betterwithmods.common.registry.bulk.recipes.MillRecipe;
import betterwithmods.util.InvUtils;
import com.google.common.collect.Lists;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.*;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;

import javax.annotation.Nonnull;
import java.util.List;
import java.util.stream.Collectors;

public class TileEntityMill extends TileBasicInventory implements ITickable, IMechanicalPower, ICrankable {

    public static final int GRIND_TIME = 200;

    public int power;
    public int grindCounter;
    public boolean blocked;
    private SoundEvent grindType;
    private boolean validateContents;
    private boolean containsIngredientsToGrind;

    public TileEntityMill() {
        this.grindCounter = 0;
        this.validateContents = true;
    }

    public boolean isActive() {
        return power > 0;
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newState) {
        return oldState.func_177230_c() != newState.func_177230_c();
    }

    public BlockMechMachines getBlock() {
        return (BlockMechMachines) this.func_145838_q();
    }

    private boolean findIfBlocked() {
        int count = 0;
        for (EnumFacing facing : EnumFacing.field_176754_o) {
            if (field_145850_b.isSideSolid(field_174879_c.func_177972_a(facing), facing.func_176734_d())) {
                count++;
            }
        }
        return count > 1;
    }

    public boolean isBlocked() {
        return blocked;
    }

    @Override
    public void func_73660_a() {
        if (this.getBlockWorld().field_72995_K)
            return;

        this.power = calculateInput();
        this.blocked = findIfBlocked();
        getBlock().setActive(field_145850_b, field_174879_c, isActive());

        validateContents();

        if (isBlocked()) {
            return;
        }

        if (isActive()) {
            if (getBlockWorld().field_73012_v.nextInt(20) == 0)
                getBlockWorld().func_184133_a(null, field_174879_c, BWSounds.STONEGRIND, SoundCategory.BLOCKS, 0.5F + getBlockWorld().field_73012_v.nextFloat() * 0.1F, 0.5F + getBlockWorld().field_73012_v.nextFloat() * 0.1F);
        }

        if (this.containsIngredientsToGrind && isActive()) {

            if (!this.getBlockWorld().field_72995_K) {
                if (grindType != null) {
                    if (this.getBlockWorld().field_73012_v.nextInt(25) < 2) {
                        getBlockWorld().func_184133_a(null, field_174879_c, grindType, SoundCategory.BLOCKS, 1F, getBlockWorld().field_73012_v.nextFloat() * 0.4F + 0.8F);
                    }
                }
            }
            this.grindCounter++;
            if (this.grindCounter > GRIND_TIME - 1) {
                grindContents();
                this.grindCounter = 0;

            }
        }
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);
        if (tag.func_74764_b("blocked"))
            this.blocked = tag.func_74767_n("blocked");
        if (tag.func_74764_b("GrindCounter"))
            this.grindCounter = tag.func_74762_e("GrindCounter");
        this.power = tag.func_74762_e("power");
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        super.func_189515_b(tag);
        tag.func_74768_a("GrindCounter", this.grindCounter);
        tag.func_74768_a("power", power);
        tag.func_74757_a("blocked", blocked);
        return tag;
    }

    @Override
    public int getInventorySize() {
        return 3;
    }

    @Override
    public void func_70296_d() {
        super.func_70296_d();
        validateContents();
        if (this.getBlockWorld() != null && !this.getBlockWorld().field_72995_K) {
            this.validateContents = true;
        }
    }

    private boolean canEject(EnumFacing facing) {
        if (field_145850_b.func_175623_d(field_174879_c.func_177972_a(facing)))
            return true;
        return !field_145850_b.func_175665_u(field_174879_c.func_177972_a(facing)) && !field_145850_b.isSideSolid(field_174879_c.func_177972_a(facing), facing.func_176734_d());
    }

    private void ejectStack(ItemStack stack) {
        List<EnumFacing> validDirections = Lists.newArrayList(EnumFacing.field_176754_o).stream().filter(this::canEject).collect(Collectors.toList());
        if (validDirections.isEmpty()) {
            blocked = true;
            return;
        }

        InvUtils.ejectStackWithOffset(getBlockWorld(), field_174879_c.func_177972_a(validDirections.get(getBlockWorld().field_73012_v.nextInt(validDirections.size()))), stack);
    }

    public double getGrindProgress() {
        return this.grindCounter / (double) GRIND_TIME;
    }

    public boolean isGrinding() {
        return this.grindCounter > 0;
    }

    private boolean grindContents() {
        if (BWRegistry.MILLSTONE.canCraft(this, inventory)) {
            NonNullList<ItemStack> output = BWRegistry.MILLSTONE.craftItem(field_145850_b, this, inventory);
            if (!output.isEmpty()) {
                for (ItemStack anOutput : output) {
                    ItemStack stack = anOutput.func_77946_l();
                    if (!stack.func_190926_b())
                        ejectStack(stack);
                }
            }
            return true;
        }
        return false;
    }

    private void validateContents() {
        SoundEvent oldGrindType = grindType;
        SoundEvent newGrindType = null;
        MillRecipe recipe = BWRegistry.MILLSTONE.getRecipe(this, inventory);
        if (recipe != null) {
            this.containsIngredientsToGrind = true;
            newGrindType = recipe.getSound();
        } else {
            this.grindCounter = 0;
            this.containsIngredientsToGrind = false;
        }
        this.validateContents = false;
        if (oldGrindType != newGrindType) {
            this.grindType = newGrindType;
        }
    }

    @Override
    public int getMechanicalOutput(EnumFacing facing) {
        return -1;
    }

    @Override
    public int getMechanicalInput(EnumFacing facing) {
        if (facing.func_176740_k().func_176720_b())
            return BWMAPI.IMPLEMENTATION.getPowerOutput(field_145850_b, field_174879_c.func_177972_a(facing), facing.func_176734_d());
        if (field_145850_b.func_175625_s(field_174879_c.func_177972_a(facing)) instanceof TileCrank) {
            return BWMAPI.IMPLEMENTATION.getPowerOutput(field_145850_b, field_174879_c.func_177972_a(facing), facing.func_176734_d());
        }
        return 0;
    }

    @Override
    public int getMaximumInput(EnumFacing facing) {
        return 1;
    }

    @Override
    public int getMinimumInput(EnumFacing facing) {
        return 0;
    }

    @Override
    public boolean hasCapability(@Nonnull Capability<?> capability, @Nonnull EnumFacing facing) {
        if (capability == CapabilityMechanicalPower.MECHANICAL_POWER)
            return true;
        return super.hasCapability(capability, facing);
    }

    @Nonnull
    @Override
    public <T> T getCapability(@Nonnull Capability<T> capability, @Nonnull EnumFacing facing) {
        if (capability == CapabilityMechanicalPower.MECHANICAL_POWER)
            return CapabilityMechanicalPower.MECHANICAL_POWER.cast(this);
        return super.getCapability(capability, facing);
    }

    @Override
    public World getBlockWorld() {
        return super.func_145831_w();
    }

    @Override
    public BlockPos getBlockPos() {
        return func_174877_v();
    }

    public boolean isUseableByPlayer(EntityPlayer player) {
        return this.getBlockWorld().func_175625_s(this.field_174879_c) == this && player.func_70092_e(this.field_174879_c.func_177958_n() + 0.5D, this.field_174879_c.func_177956_o() + 0.5D, this.field_174879_c.func_177952_p() + 0.5D) <= 64.0D;
    }
}
